function Flash(block);
%level-2 MATLAB file

%MSFUNTMPL A Template for a MATLAB S-Function
%   The MATLAB S-function is written as a MATLAB function with the
%   same name as the S-function. Replace 'msfuntmpl' with the name
%   of your S-function.  
%
%   It should be noted that the MATLAB S-function is very similar
%   to Level-2 C-Mex S-functions. You should be able to get more 
%   information for each of the block methods by referring to the
%   documentation for C-Mex S-functions.
%  
%   Copyright 2003-2010 The MathWorks, Inc.
%   $Revision: 1.1.6.22 $  
  
%
% The setup method is used to setup the basic attributes of the
% S-function such as ports, parameters, etc. Do not add any other
% calls to the main body of the function.  

setup(block)

%endfunction

% Function: setup ===================================================
% Abstract:
%   Set up the S-function block's basic characteristics such as:
%   - Input ports
%   - Output ports
%   - Dialog parameters
%   - Options
% 
%   Required         : Yes
%   C-Mex counterpart: mdlInitializeSizes

function setup(block);
  
  block.NumDialogPrms  = 0;
  
  %% Register number of input and output ports
  % the input ports are flow, composition and temperature
  block.NumInputPorts  = 3;
  %Outlet will be composition and flow in the different phases and total
  %and moles of glycerol in tank
  block.NumOutputPorts = 6;
  
  

  %% Setup functional port properties to dynamically
  %% inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  
  %Flow input
  %block.InputPort(1).Complexity   = 'Real'; 
  block.InputPort(1).Dimensions        = 1;
  %block.InputPort(1).SamplingMode = fd;
  block.InputPort(1).DirectFeedthrough = false;
  
  %Methanol fraction
  block.InputPort(2).Dimensions        = 1;
  %block.InputPort(2).SamplingMode = fd;
  block.InputPort(2).DirectFeedthrough = false;
  
  %Temperature
  block.InputPort(3).Dimensions        = 1;
  %block.InputPort(2).SamplingMode = fd;
  block.InputPort(3).DirectFeedthrough = false;
  
  % Fraction methanol in top
  block.OutputPort(1).Dimensions       = 1;
  % Flow in top
  block.OutputPort(2).Dimensions       = 1;
  % Fraction methanol in bottom
  block.OutputPort(3).Dimensions       = 1;
  % Flow in bottom
  block.OutputPort(4).Dimensions       = 1;
  % Mole glycerol in tank
  block.OutputPort(5).Dimensions       = 1;
  % Mole in tank
  block.OutputPort(6).Dimensions       = 1;
  
  
  %% Set block sample time to inherited
  block.SampleTimes = [0 0]; 
  
 block.NumContStates = 2; 
  
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
 % block.SimStateCompliance = 'DefaultSimState';

  %% Register methods
  %This needs to include all the function blocks below for them to be
  %called on
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSamplingMode',@SetInpPortFrameData);
  block.RegBlockMethod('Derivatives',             @Derivatives);

function SetInpPortFrameData(block, idx, fd)
 
block.InputPort(idx).SamplingMode = fd;
for i = 1:6
    block.OutputPort(i).SamplingMode = fd;
end
 
function InitConditions(block);

  %% Initialize Dwork

 %total amount of moles, moles of glycerol
 load('initFlash') 

  block.ContStates.Data=x0;
  
%endfunction

function Output(block);

%State variables, taken from the ContStates block
n=block.ContStates.Data(1); % Number of moles in flash
n1=block.ContStates.Data(2); % Number of moles of glycerol in flash

%Disturbances
F=block.InputPort(1).Data; % Feed flow mol/s
z=block.InputPort(2).Data; % Feed composition, fraction methanol
T=block.InputPort(3).Data; % Temperature K

% Given parameters
Vtank=10; % Tank volume m^3 
VLs=Vtank/2; % Want 50% liquid
kg=0.1;  % Valve position
kl=100*10^3; % Controller gain
R=8.134; % Gas constant
p0=1*10^5; % Atmospheric pressure downstream
x1=n1/n; % Fraction of glycerol in liquid, since almost all glycerol will be in liquid phase this assumptions is ok
x2=1-x1; % Fraction of methanol in liquid 

Vl2=4.05800*10^-5; % Molar volume of methanol m^3/mol from Dippr
Vl1=7.32027*10^-5; % Molar volume of glycerol m^3/mol from Dippr
Vm=((1-x2)*Vl1+x2*Vl2); % Molar volume in liquid phase 

%Glycerol, Antoine coefficient
 A1=9.9986*10^1; 
 B1=-1.3808*10^4;
 C1=-1.0088*10^1;
 D1=3.5712*10^-19;
 E1=6*10^0;

Psat1=exp(A1+B1/T+C1*log(T)+D1*T^E1); % Saturation pressure Pa

%Methanol, Antoine coefficient
 A2=8.2718*10^1;
 B2=-6.9045*10^3;
 C2=-8.8622*10^0;
 D2=7.4664*10^-6;
 E2=2*10^0;

Psat2=exp(A2+B2/T+C2*log(T)+D2*T^E2); % Saturation pressure Pa

%Total pressure Pa
P=x1*Psat1+(1-x1)*Psat2; 

% Vapour compostion from Raoults law
y1=(1-x2)*(Psat1)/P; %assumed ideal the fraction of glycerol in vapour
y2=x2*(Psat2)/P; %assmued ideal the fraction of methanol in vapor

%Liquid volume m^3
VL=(n-P*Vtank/(R*T))/(1/Vm-P/(R*T)); 
%Vapour volume m^3
VG=Vtank-VL;

% The outputs, give the mole fraction x2 of Methanol
block.OutputPort(1).Data=y2; 
% Give the molar flow V
block.OutputPort(2).Data=kg*(P-p0); 
% Mole fraction methanol
block.OutputPort(3).Data=x2;
%Molar flow of L
block.OutputPort(4).Data=kl*(VL-VLs);
%See the states
block.OutputPort(5).Data=n1;
block.OutputPort(6).Data=n;

  
function Derivatives(block);

%State variables, taken from the ContStates block
n=block.ContStates.Data(1); % Number of moles in flash
n1=block.ContStates.Data(2); % Number of moles of glycerol in flash

%Disturbances
F=block.InputPort(1).Data; % Feed flow mol/s
z=block.InputPort(2).Data; % Feed composition, fraction methanol
T=block.InputPort(3).Data; % Temperature K

% Given parameters
Vtank=10; %m^3 
VLs=Vtank/2; %want 50% full tank
kg=0.1;  % Valve position
kl=100*10^3; % Controller gain
R=8.134; % Gas constant
p0=1*10^5; % Atmospheric pressure downstream
x1=n1/n; % Fraction of glycerol in liquid, since almost all glycerol will be in liquid phase this assumptions is ok
x2=1-x1; % Fraction of methanol in liquid 

Vl2=4.05800*10^-5; % Molar volume of methanol m^3/mol from Dippr
Vl1=7.32027*10^-5; % Molar volume of glycerol m^3/mol from Dippr
Vm=((1-x2)*Vl1+x2*Vl2); % Molar volume in liquid phase 

%Glycerol, Antoine coefficient
 A1=9.9986*10^1; 
 B1=-1.3808*10^4;
 C1=-1.0088*10^1;
 D1=3.5712*10^-19;
 E1=6*10^0;

Psat1=exp(A1+B1/T+C1*log(T)+D1*T^E1); % Saturation pressure Pa

%Methanol, Antoine coefficient
 A2=8.2718*10^1;
 B2=-6.9045*10^3;
 C2=-8.8622*10^0;
 D2=7.4664*10^-6;
 E2=2*10^0;

Psat2=exp(A2+B2/T+C2*log(T)+D2*T^E2); % Saturation pressure Pa

%Total pressure Pa
P=x1*Psat1+(1-x1)*Psat2; 

% Vapour compostion from Raoults law
y1=(1-x2)*(Psat1)/P; %assumed ideal the fraction of glycerol in vapour
y2=x2*(Psat2)/P; %assmued ideal the fraction of methanol in vapor

%Liquid volume m^3
VL=(n-P*Vtank/(R*T))/(1/Vm-P/(R*T)); 
%Vapour volume m^3
VG=Vtank-VL;

% Vapour flow
V=kg*(P-p0);
% Liquid flow
L=kl*(VL-VLs);

%The derivatives
dxdt(1)=F-V-L;
dxdt(2)=F*(1-z)-V*(1-y2)-L*(1-x2);

block.Derivatives.Data = dxdt;

