function Settlingtankmol(block);
%level-2 MATLAB file

%MSFUNTMPL A Template for a MATLAB S-Function
%   The MATLAB S-function is written as a MATLAB function with the
%   same name as the S-function. Replace 'msfuntmpl' with the name
%   of your S-function.  
%
%   It should be noted that the MATLAB S-function is very similar
%   to Level-2 C-Mex S-functions. You should be able to get more 
%   information for each of the block methods by referring to the
%   documentation for C-Mex S-functions.
%  
%   Copyright 2003-2010 The MathWorks, Inc.
%   $Revision: 1.1.6.22 $  
  
%
% The setup method is used to setup the basic attributes of the
% S-function such as ports, parameters, etc. Do not add any other
% calls to the main body of the function.  

setup(block)

%endfunction

% Function: setup ===================================================
% Abstract:
%   Set up the S-function block's basic characteristics such as:
%   - Input ports
%   - Output ports
%   - Dialog parameters
%   - Options
% 
%   Required         : Yes
%   C-Mex counterpart: mdlInitializeSizes

function setup(block);
  
  block.NumDialogPrms  = 0;
  
  %% Register number of input and output ports
  % The input ports are inlet flow, outlet flow and composition
  block.NumInputPorts  = 3;
  % The outlet will be the composition and molar hold up
  block.NumOutputPorts = 2;
  
  

  %% Setup functional port properties to dynamically
  %% inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  
  %composition input
  %block.InputPort(1).Complexity   = 'Real'; 
  block.InputPort(1).Dimensions        = 6;
  %block.InputPort(1).SamplingMode = fd;
  block.InputPort(1).DirectFeedthrough = false;
  
  %Inlet flow
  block.InputPort(2).Dimensions        = 1;
  %block.InputPort(2).SamplingMode = fd;
  block.InputPort(2).DirectFeedthrough = false;
  
  %Outlet flow
  block.InputPort(3).Dimensions        = 1;
  %block.InputPort(3).SamplingMode = fd;
  block.InputPort(3).DirectFeedthrough = false;
 
  % Outlet composition
  block.OutputPort(1).Dimensions       = 6;
  % Molar hold up
  block.OutputPort(2).Dimensions       = 1;
 
  
  %% Set block sample time to inherited
  block.SampleTimes = [0 0]; %by having 0 0 the steps are taken as Simulink wants
 % Nominal composition of five components and molar hold up
 block.NumContStates = 6; %the number of differential equation? or  is it conditions
  
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
 % block.SimStateCompliance = 'DefaultSimState';

  %% Register methods
  %This needs to include all the function blocks below for them to be
  %called on
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSamplingMode',@SetInpPortFrameData);
  block.RegBlockMethod('Derivatives',             @Derivatives);
  


function SetInpPortFrameData(block, idx, fd)
 
block.InputPort(idx).SamplingMode = fd;
for i = 1:2
    block.OutputPort(i).SamplingMode = fd;
end
 

function InitConditions(block);

  %% Initialize Dwork
  
 load(['initsettling','.mat']);
 
  block.ContStates.Data=x0;
  

function Output(block);

% Disturbances, TG, DG, MG, ME, GL, MEOH
x1=block.InputPort(1).Data(1);
x2=block.InputPort(1).Data(2);
x3=block.InputPort(1).Data(3);
x4=block.InputPort(1).Data(4);
x5=block.InputPort(1).Data(5);
x6=block.InputPort(1).Data(6);

% Inlet and outlet flows
F=block.InputPort(2).Data;
Fout=block.InputPort(3).Data;

%Molar holpup
M=block.ContStates.Data(6);

%State variables for the composition
x1out=block.ContStates.Data(1);
x2out=block.ContStates.Data(2);
x3out=block.ContStates.Data(3);
x4out=block.ContStates.Data(4);
x5out=block.ContStates.Data(5);
x6out=(1-x1out-x2out-x3out-x4out-x5out);

%Output composition and hold up
block.OutputPort(1).Data(1)=x1out;
block.OutputPort(1).Data(2)=x2out;
block.OutputPort(1).Data(3)=x3out;
block.OutputPort(1).Data(4)=x4out;
block.OutputPort(1).Data(5)=x5out;
block.OutputPort(1).Data(6)=x6out;
block.OutputPort(2).Data=M;

  
function Derivatives(block);

%TG=y(1), DG=y(2), MG=y(3), ME=y(4), GL=y(5), MeOH=y(6)

%Disturbances, composition, TG, DG, MG ME GL MEOH
x1=block.InputPort(1).Data(1);
x2=block.InputPort(1).Data(2);
x3=block.InputPort(1).Data(3);
x4=block.InputPort(1).Data(4);
x5=block.InputPort(1).Data(5);
x6=block.InputPort(1).Data(6);

% Flows
F=block.InputPort(2).Data;
Fout=block.InputPort(3).Data;

% Holdup
M=block.ContStates.Data(6);

% Outlet composition
x1out=block.ContStates.Data(1);
x2out=block.ContStates.Data(2);
x3out=block.ContStates.Data(3);
x4out=block.ContStates.Data(4);
x5out=block.ContStates.Data(5);
x6out=(1-x1out-x2out-x3out-x4out-x5out);

% Change in molefraction of component, and molar hold up
dhdt(1)=(x1*F-x1out*Fout)/M;
dhdt(2)=(x2*F-x2out*Fout)/M;
dhdt(3)=(x3*F-x3out*Fout)/M;
dhdt(4)=(x4*F-x4out*Fout)/M;
dhdt(5)=(x5*F-x5out*Fout)/M;
dhdt(6)=F-Fout;
block.Derivatives.Data = dhdt;

