function Split(block);
%level-2 MATLAB file

%MSFUNTMPL A Template for a MATLAB S-Function
%   The MATLAB S-function is written as a MATLAB function with the
%   same name as the S-function. Replace 'msfuntmpl' with the name
%   of your S-function.  
%
%   It should be noted that the MATLAB S-function is very similar
%   to Level-2 C-Mex S-functions. You should be able to get more 
%   information for each of the block methods by referring to the
%   documentation for C-Mex S-functions.
%  
%   Copyright 2003-2010 The MathWorks, Inc.
%   $Revision: 1.1.6.22 $  
  
%
% The setup method is used to setup the basic attributes of the
% S-function such as ports, parameters, etc. Do not add any other
% calls to the main body of the function.  

setup(block)

%endfunction

% Function: setup ===================================================
% Abstract:
%   Set up the S-function block's basic characteristics such as:
%   - Input ports
%   - Output ports
%   - Dialog parameters
%   - Options
% 
%   Required         : Yes
%   C-Mex counterpart: mdlInitializeSizes

function setup(block);
  
  block.NumDialogPrms  = 0;
  
  %% Register number of input and output ports
  % the input ports are flow, temperature, and composition
  block.NumInputPorts  = 3;
  %by having the same output the model can be expanded to several CSTR in
  %series
  block.NumOutputPorts = 6;
  
  

  %% Setup functional port properties to dynamically
  %% inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  
  % Composition input
  block.InputPort(1).Dimensions        = 6;
  block.InputPort(1).DirectFeedthrough = false;
  
  % Flow
  block.InputPort(2).Dimensions        = 1;
  block.InputPort(2).DirectFeedthrough = false;
  
  % Temperature
  block.InputPort(3).Dimensions        = 1;
  block.InputPort(3).DirectFeedthrough = false;
  
  %The lighter, biodiesel, phase
  block.OutputPort(1).Dimensions       = 6;
  block.OutputPort(2).Dimensions       = 1;
  block.OutputPort(3).Dimensions       = 1;
  
  %the heavier, glycerol, phase
  block.OutputPort(4).Dimensions       = 6;
  block.OutputPort(5).Dimensions       = 1;
  block.OutputPort(6).Dimensions       = 1;
 
  
  %% Set block sample time to inherited
  block.SampleTimes = [0 0]; %by having 0 0 the steps are taken as Simulink wants
  block.NumContStates = 0; 
  
 

  %% Register methods
  %This needs to include all the function blocks below for them to be
  %called on
   
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSamplingMode',@SetInpPortFrameData);


function SetInpPortFrameData(block, idx, fd)
 
block.InputPort(idx).SamplingMode = fd;
for i = 1:6
    block.OutputPort(i).SamplingMode = fd;
end
 


function Output(block);

%Disturbances
y=block.InputPort(1).Data;
F=block.InputPort(2).Data;
T=block.InputPort(3).Data;

% How much in the top flow of the inflow 
split=[0.05 0.05 0.05 1 0 0.4];

% The rest goes in the bottom
split2=1-split;

% Molefraction of original in top
block.OutputPort(1).Data=((y'.*split).*F)/F;
% Flow in top
block.OutputPort(2).Data=(sum((y'.*split).*F));
% Temperature in top phase
block.OutputPort(3).Data=T;
% Molefraction of original in bottom
block.OutputPort(4).Data=((y'.*split2).*F)/(F);
% Flow in bottom
block.OutputPort(5).Data=(sum((y'.*split2).*F)); 
% Temperature in bottom phase
block.OutputPort(6).Data=T;

