
function bioreactor(block);
%level-2 MATLAB file

%MSFUNTMPL A Template for a MATLAB S-Function
%   The MATLAB S-function is written as a MATLAB function with the
%   same name as the S-function. Replace 'msfuntmpl' with the name
%   of your S-function.  
%
%   It should be noted that the MATLAB S-function is very similar
%   to Level-2 C-Mex S-functions. You should be able to get more 
%   information for each of the block methods by referring to the
%   documentation for C-Mex S-functions.
%  
%   Copyright 2003-2010 The MathWorks, Inc.
%   $Revision: 1.1.6.22 $  
  
%
% The setup method is used to setup the basic attributes of the
% S-function such as ports, parameters, etc. Do not add any other
% calls to the main body of the function.  

setup(block)

%endfunction

% Function: setup ===================================================
% Abstract:
%   Set up the S-function block's basic characteristics such as:
%   - Input ports
%   - Output ports
%   - Dialog parameters
%   - Options
% 
%   Required         : Yes
%   C-Mex counterpart: mdlInitializeSizes

function setup(block);
  
  block.NumDialogPrms  = 0;
  
  %% Register number of input and output ports
  % the input ports are flow, temperature, and composition
  block.NumInputPorts  = 3;
  % The outputs will be the same as the input
  block.NumOutputPorts = 3;
  
  %% Setup functional port properties to dynamically
  %% inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  
  % Mole fraction input
  block.InputPort(1).Dimensions        = 6;
  block.InputPort(1).DirectFeedthrough = false;
  
  % Molarflow
  block.InputPort(2).Dimensions        = 1;
  block.InputPort(2).DirectFeedthrough = false;
  
  % Temperature
  block.InputPort(3).Dimensions        = 1;
  block.InputPort(3).DirectFeedthrough = false;
  
  % Mole fraction of the six components in the ten reactors, molarflow, temperature
  block.OutputPort(1).Dimensions       = 60;
  block.OutputPort(2).Dimensions       = 1;
  block.OutputPort(3).Dimensions       = 1;
  
  %% Set block sample time to inherited
  block.SampleTimes = [0 0]; 
  block.NumContStates = 60; 
  
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
 % block.SimStateCompliance = 'DefaultSimState';

  %% Register methods
  %This needs to include all the function blocks below for them to be
  %called on

  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSamplingMode',@SetInpPortFrameData);
  block.RegBlockMethod('Derivatives',             @Derivatives);
  
%endfunction

function SetInpPortFrameData(block, idx, fd)
block.InputPort(idx).SamplingMode = fd;
for i = 1:3
    block.OutputPort(i).SamplingMode = fd;
end



function InitConditions(block);

  %% Initialize Dwork
 % The states for all the molefractions in all the reactors
 load('init1.mat')
 
  block.ContStates.Data=y0;
  
%endfunction

function Output(block);

%State variables
y=block.ContStates.Data(1:60);

%Disturbances
zF=block.InputPort(1).Data;
Fin=block.InputPort(2).Data;
Tin=block.InputPort(3).Data;

%Outputvariables
block.OutputPort(1).Data=y;
block.OutputPort(2).Data=Fin;
block.OutputPort(3).Data=Tin;

function Derivatives(block);

%TG=y(1), DG=y(2), MG=y(3), ME=y(4), GL=y(5), MeOH=y(6)
%Transesterification reactions
    %TG+MeOH->(k(1))DG+ME
    %DG+ME->(k(2))TG+MeOH
    %DG+MeOH->(k(3))MG+ME
    %MG+ME->(k(4))DG+MeOH
    %MG+MeOH->(k(5))GL+ME
    %GL+ME->(k(6))MG+MeOH
    
%State variables, taken from the ContStates block
y=block.ContStates.Data(1:60);

% Disturbances in inlet
zF=block.InputPort(1).Data;
Fin=block.InputPort(2).Data;
Tin=block.InputPort(3).Data;

%Outputvariables
block.OutputPort(1).Data=y;
block.OutputPort(2).Data=Fin;
block.OutputPort(3).Data=Tin;  

%The activation energy for each reaction, given in cal/mol
%par.Ea(1)=13145*4.2; %activation energy at 60 degree celsius part 1 J/mol
%par.Ea(2)=9932*4.2; %activation energy at 60 degree celsius part 2
%par.Ea(3)=19860*4.2; %activation energy at 60 degree celsius part 3
%par.Ea(4)=14639*4.2; %activation energy at 60 degree celsius part 4
%par.Ea(5)=-6421*4.2; %activation energy at 60 degree celsius part 5
%par.Ea(6)=-9588*4.2; %activation energy at 60 degree celsius part 6

% The Arrhenius constant L/(sec)mol, left outside due to stiffness of the
% problem and possibility to solve
%par.A(1)=(3.89*10^7)/60; 
%par.A(2)=(5.74*10^5)/60; 
%par.A(3)=(5.82*10^12)/60;
%par.A(4)=(9.78*10^9)/60;
%par.A(5)=(1.1*10^(-5))/60;
%par.A(6)=(2.29*10^(-9))/60;

% The universal gas constant
%par.R=8.314; %J/K*mol
 
% Constant reaction parameters
 par.k(1)=0.049/60;
 par.k(2)=0.102/60;
 par.k(3)=0.218/60;
 par.k(4)=1.28/60;
 par.k(5)=0.239/60;
 par.k(6)=0.007/60;
 
%par.k(1)=par.A(1)*exp(-par.Ea(1)/(par.R*(block.InputPort(3).Data)+273));
%par.k(2)=par.A(2)*exp(-par.Ea(2)/(par.R*(block.InputPort(3).Data)+273));
%par.k(3)=par.A(3)*exp(-par.Ea(3)/(par.R*(block.InputPort(3).Data)+273));
%par.k(4)=par.A(4)*exp(-par.Ea(4)/(par.R*(block.InputPort(3).Data)+273));
%par.k(5)=par.A(5)*exp(-par.Ea(5)/(par.R*(block.InputPort(3).Data)+273));
%par.k(6)=par.A(6)*exp(-par.Ea(6)/(par.R*(block.InputPort(3).Data)+273));

%Residence time in each reactor
teach=180;

% Volume each tank, from total volume of plug flow reactor
Veachl=1451.9;
 
% Model
% For the first tank
dydt(1)=(1/teach)*(zF(1)-y(1))+((Fin*teach)/Veachl)*(-par.k(1)*y(1)*y(6)+par.k(2)*y(2)*y(4));
dydt(2)=(1/teach)*(zF(2)-y(2))+((Fin*teach)/Veachl)*(par.k(1)*y(1)*y(6)-par.k(2)*y(2)*y(4)-par.k(3)*y(2)*y(6)+par.k(4)*y(3)*y(4));
dydt(3)=(1/teach)*(zF(3)-y(3))+((Fin*teach)/Veachl)*(par.k(3)*y(2)*y(6)-par.k(4)*y(3)*y(4)-par.k(5)*y(3)*y(6)+par.k(6)*y(4)*y(5));
dydt(4)=(1/teach)*(zF(4)-y(4))+((Fin*teach)/Veachl)*(par.k(1)*y(1)*y(6)-par.k(2)*y(2)*y(4)+par.k(3)*y(2)*y(6)-par.k(4)*y(3)*y(4)+par.k(5)*y(3)*y(6)-par.k(6)*y(4)*y(5));
dydt(5)=(1/teach)*(zF(5)-y(5))+((Fin*teach)/Veachl)*(par.k(5)*y(3)*y(6)-par.k(6)*y(4)*y(5));
dydt(6)=(1/teach)*(zF(6)-y(6))+((Fin*teach)/Veachl)*(-par.k(1)*y(1)*y(6)+par.k(2)*y(2)*y(4)-par.k(3)*y(2)*y(6)+par.k(4)*y(3)*y(4)-par.k(5)*y(3)*y(6)+par.k(6)*y(4)*y(5));

% For the following reactors
for i=7:6:55
dydt(i)=(1/teach)*(y(i-6)-y(i))+((Fin*teach)/Veachl)*(-par.k(1)*y(i)*y(i+5)+par.k(2)*y(i+1)*y(i+3));
dydt(i+1)=(1/teach)*(y(i-5)-y(i+1))+((Fin*teach)/Veachl)*(par.k(1)*y(i)*y(i+5)-par.k(2)*y(i+1)*y(i+3)-par.k(3)*y(i+1)*y(i+5)+par.k(4)*y(i+2)*y(i+3));
dydt(i+2)=(1/teach)*(y(i-4)-y(i+2))+((Fin*teach)/Veachl)*(par.k(3)*y(i+1)*y(i+5)-par.k(4)*y(i+2)*y(i+3)-par.k(5)*y(i+2)*y(i+5)+par.k(6)*y(i+3)*y(i+4));
dydt(i+3)=(1/teach)*(y(i-3)-y(i+3))+((Fin*teach)/Veachl)*(par.k(1)*y(i)*y(i+5)-par.k(2)*y(i+1)*y(i+3)+par.k(3)*y(i+1)*y(i+5)-par.k(4)*y(i+2)*y(i+3)+par.k(5)*y(i+2)*y(i+5)-par.k(6)*y(i+3)*y(i+4));
dydt(i+4)=(1/teach)*(y(i-2)-y(i+4))+((Fin*teach)/Veachl)*(par.k(5)*y(i+2)*y(i+5)-par.k(6)*y(i+3)*y(i+4));
dydt(i+5)=(1/teach)*(y(i-1)-y(i+5))+((Fin*teach)/Veachl)*(-par.k(1)*y(i)*y(i+5)+par.k(2)*y(i+1)*y(i+3)-par.k(3)*y(i+1)*y(i+5)+par.k(4)*y(i+2)*y(i+3)-par.k(5)*y(i+2)*y(i+5)+par.k(6)*y(i+3)*y(i+4));
end
block.Derivatives.Data = dydt;
